local player = require "player"
local asteroidController = require "asteroid"

local module = {}

function module.updateCounters()
    if player.bulletCooldown < 35 then
        player.bulletCooldown = player.bulletCooldown+1
    end
    
    if player.vulnerable > 0 then
        player.vulnerable = player.vulnerable-1
    end
end

function module.getInput(dt)
    if player.lives > 0 then
        --get rotation
        if love.keyboard.isDown("a") then
            player.rotMoment = player.rotMoment - player.rotSpeed * dt

        elseif love.keyboard.isDown("d") then
            player.rotMoment = player.rotMoment + player.rotSpeed * dt
        end
        
        --calculate the x and y components of the direction vector
        player.xRot = (math.cos(player.rot * math.pi / 360))
        player.yRot = (math.sin(player.rot * math.pi / 360))
        
        --prevent overflow
        if math.abs(player.rot * math.pi / 360) - (math.pi * 2) > 0.01 then
            player.rot = 0
        end
        
        --get speed
        if love.keyboard.isDown("w") then
            player.faster()
        
        elseif love.keyboard.isDown("s") then
            player.slower()
        end
        
        if love.keyboard.isDown("x") then
            player.brake()
        end
        
        if love.keyboard.isDown("space") then
            if player.bulletCooldown == 35 then
                player.fire()
            end
        end
    end

    if love.keyboard.isDown("escape") then
        love.event.quit()
    end
    
    if love.keyboard.isDown("r") then
        resetGame()
    end
end

function module.moveBullets(dt)
    for i,b in ipairs(player.bullets) do
        b.x = b.x + b.xMoment
        b.y = b.y + b.yMoment
        b.age = b.age + 1
        
        --remove bullets if they are off the screen
        if b.y < -5 or b.y > love.graphics.getHeight()+5 then
            table.remove(player.bullets, i)
        end
        
        if b.x < -5 or b.x > love.graphics.getWidth()+5 then
            table.remove(player.bullets, i)
        end
        
        --remove old bullets
        if b.age > 5000 then
            table.remove(player.bullets, i)
        end
    end
end

function module.drawBullets()
    for _,b in pairs(player.bullets) do
        love.graphics.rectangle("line", b.x, b.y, 2, 2)
    end
end

function module.collision()
    for i,a in ipairs(asteroidController.asteroids) do
        --check collision with ship
        local tempDist = module.distance(player.x, player.y, a.x, a.y)
        if player.vulnerable == 0 and tempDist < a.size then
            table.remove(asteroidController.asteroids, i)
            player.lives = player.lives-1
            player.vulnerable = 100
            
            if player.lives < 1 then
                gameOver()
                return
            end
        end
        
        for j, b in ipairs(player.bullets) do
            --check collision with bullets
            tempDist = module.distance(b.x, b.y, a.x, a.y)
            if tempDist < a.size then
                table.remove(player.bullets, j)
                --remove if it is a triangle
                if a.sides == 3 then
                    table.remove(asteroidController.asteroids, i)
                
                --reduce sides otherwise
                else
                    a.sides = a.sides-1
                end
                break
            end
        end        
    end
end

function module.distance(x1, y1, x2, y2)
    return math.sqrt( (x1-x2)*(x1-x2)  + (y1-y2)*(y1-y2) )
end

function module.resetGame()
    --reset player position
    player.x = 400
    player.y = 300
    player.rot = -180
    
    player.xMoment = 0
    player.yMoment = 0
    player.rotMoment = 0
    
    player.lives = 3

    --clear bullets
    for i,v in ipairs(player.bullets) do
        player.bullets[i] = nil
    end
    
    --clear asteroids
    for i,a in ipairs(asteroidController.asteroids) do
        asteroidController.asteroids[i] = nil
    end
    
    love.load()
end

function module.gameOver()
    --reset player position
    player.x = 400
    player.y = 300
    player.rot = -180
    
    player.xMoment = 0
    player.yMoment = 0
    player.rotMoment = 0
    
    --clear bullets
    for i,v in ipairs(player.bullets) do
        player.bullets[i] = nil
    end
    
    --clear asteroids
    for i,a in ipairs(asteroidController.asteroids) do
        asteroidController.asteroids[i] = nil
    end
end

return module